#!/usr/bin/env python
"""
Data Handler (step 1)
"""

import glob
import os
from pathlib import PurePath

import numpy as np
from loguru import logger
from qtpy.QtWidgets import QFileDialog, QListWidgetItem

from ibeatles import DataType

# MVP-based widget import
from ibeatles.app.presenters.time_spectra_presenter import TimeSpectraPresenter

# Import backend function from new core module
from ibeatles.core.io.data_loading import get_time_spectra_filename
from ibeatles.utilities.file_handler import (
    FileHandler,
    get_list_of_most_dominant_extension_from_folder,
)
from ibeatles.utilities.gui_handler import GuiHandler
from ibeatles.utilities.load_files import LoadFiles
from ibeatles.utilities.status_message_config import (
    StatusMessageStatus,
    show_status_message,
)
from ibeatles.utilities.system import is_os_mac

TIME_SPECTRA_NAME_FORMAT = "*_Spectra.txt"


class DataHandler:
    user_canceled = False

    def __init__(self, parent=None, data_type=None):
        self.parent = parent

        self.time_spectra_presenter = None

        if data_type is None:
            o_gui = GuiHandler(parent=parent)
            data_type = o_gui.get_active_tab()

        self.data_type = data_type

        self.list_ui = {
            "sample": {
                "list": self.parent.ui.list_sample,
                "folder": self.parent.ui.sample_folder,
                "time_spectra": {
                    "filename": self.parent.ui.time_spectra,
                    "folder": self.parent.ui.time_spectra_folder,
                },
            },
            "ob": {
                "list": self.parent.ui.list_open_beam,
                "folder": self.parent.ui.open_beam_folder,
            },
            "normalized": {
                "list": self.parent.ui.list_normalized,
                "folder": self.parent.ui.normalized_folder,
                "time_spectra": {
                    "filename": self.parent.ui.time_spectra_2,
                    "folder": self.parent.ui.time_spectra_folder_2,
                },
            },
            "time_spectra": {
                "text": self.parent.ui.time_spectra,
                "text2": self.parent.ui.time_spectra_2,
                "folder": self.parent.ui.time_spectra_folder,
                "folder2": self.parent.ui.time_spectra_folder_2,
            },
        }

    def raises_error(self):
        raise ValueError

    def select_folder(self):
        if is_os_mac():
            _folder = str(
                QFileDialog.getExistingDirectory(
                    caption="Select {} folder".format(self.data_type),
                    directory=os.path.dirname(self.parent.default_path[self.data_type]),
                    options=QFileDialog.ShowDirsOnly,
                )
            )
        else:
            _folder = str(
                QFileDialog.getExistingDirectory(
                    caption="Select {} folder".format(self.data_type),
                    directory=os.path.dirname(self.parent.default_path[self.data_type]),
                    options=QFileDialog.DontUseNativeDialog,
                )
            )
        return _folder

    def import_files_from_folder(self, folder="", extension=None):
        logger.info(f"importing files from folder with extension: {extension =}")
        if folder == "":
            self.user_canceled = True
            return False

        if not extension:
            # load the most dominant files
            logger.info("loading the most dominant extension")
            list_of_files, ext = get_list_of_most_dominant_extension_from_folder(folder=folder)
            logger.info(f"\textension: {ext}")
            logger.info(f"\tnbr_files: {len(list_of_files)}")

        elif type(extension) is list:
            for _ext in extension:
                list_of_files = self.get_list_of_files(folder=folder, file_ext=_ext)
                if list_of_files:
                    break
        else:
            list_of_files = self.get_list_of_files(folder=folder, file_ext=extension)

        if not list_of_files:
            logger.info(f"Folder {folder} is empty or does not contain the right file format!")
            show_status_message(
                parent=self.parent,
                message="Folder selected is empty or contains the wrong file formats!",
                status=StatusMessageStatus.error,
                duration_s=5,
            )
            return False

        logger.info(f" len(list_of_files) = {len(list_of_files)}")
        if len(list_of_files) > 2:
            logger.info(f"  list_of_files[0] = {list_of_files[0]}")
            logger.info("    ...")
            logger.info(f"  list_of_files[-1] = {list_of_files[-1]}")
        else:
            logger.info(f"  list_of_files = {list_of_files}")
        self.load_files(list_of_files)
        return True

    def import_time_spectra(self):
        if self.parent.data_metadata[self.data_type]["data"] is not None:
            if (self.data_type == DataType.sample) or (self.data_type == DataType.normalized):
                self.load_time_spectra()

    def get_list_of_files(self, folder="", file_ext=".fits"):
        """list of files in that folder with that extension"""
        file_regular_expression = os.path.join(folder, "*" + file_ext)
        list_of_files = glob.glob(file_regular_expression)
        list_of_files.sort()
        return list_of_files

    def load_files(self, list_of_files):
        logger.info("Loading files")
        image_type = DataHandler.get_image_type(list_of_files)
        logger.info(f" image type: {image_type}")
        o_load_image = LoadFiles(parent=self.parent, image_ext=image_type, list_of_files=list_of_files)
        self.populate_list_widget(o_load_image)
        self.record_data(o_load_image)

    def record_data(self, o_load_image):
        self.parent.list_files[self.data_type] = o_load_image.list_of_files
        self.parent.data_metadata[self.data_type]["folder"] = o_load_image.folder
        self.parent.data_metadata[self.data_type]["data"] = np.array(o_load_image.image_array)

    def get_time_spectra_file(self):
        if self.data_type == "normalized":
            folder = self.parent.default_path["normalized"]
        else:
            folder = self.parent.default_path["sample"]
        return get_time_spectra_filename(folder)

    def browse_file_name(self):
        [file_name, _] = QFileDialog.getOpenFileName(
            caption="Select the Time Spectra File",
            directory=self.parent.default_path[self.data_type],
            filter="Txt ({});;All (*.*)".format(TIME_SPECTRA_NAME_FORMAT),
        )
        if file_name:
            return file_name

    def load_time_spectra(self, time_spectra_file=None):
        logger.info("running load_time_spectra method")
        if time_spectra_file is None:
            time_spectra_file = self.get_time_spectra_file()
            if not time_spectra_file:
                time_spectra_file = self.browse_file_name()

        if time_spectra_file is None:
            logger.info("User cancel browsing for time_spectra!")
            show_status_message(
                parent=self.parent,
                message="User cancel browsing for time spectra file!",
                status=StatusMessageStatus.warning,
                duration_s=5,
            )
            return

        logger.info(f"time_spectra_file: {time_spectra_file}")
        self.parent.data_metadata[self.data_type]["time_spectra"]["filename"] = time_spectra_file

        if self.time_spectra_presenter is None:
            self.time_spectra_presenter = TimeSpectraPresenter(self.parent)

        distance_source_detector_m = float(self.parent.ui.distance_source_detector.text())
        detector_offset = float(self.parent.ui.detector_offset.text())

        try:
            self.time_spectra_presenter.load_data(time_spectra_file, distance_source_detector_m, detector_offset)
            self.save_tof_and_lambda_array()
            self.print_time_spectra_filename(time_spectra_file)
        except Exception as e:
            logger.error(f"Error loading time spectra: {str(e)}")
            show_status_message(
                parent=self.parent,
                message=f"Error loading time spectra: {str(e)}",
                status=StatusMessageStatus.error,
                duration_s=5,
            )

    def save_tof_and_lambda_array(self):
        time_spectra_data = self.time_spectra_presenter.model.get_data()
        tof_array = time_spectra_data["tof_array"]
        lambda_array = time_spectra_data["lambda_array"]

        if self.data_type == "sample":
            tof_key = "data"
            lambda_key = "lambda"
        elif self.data_type == "normalized":
            tof_key = "normalized_data"
            lambda_key = "normalized_lambda"

        self.parent.data_metadata["time_spectra"][tof_key] = tof_array
        self.parent.data_metadata["time_spectra"][lambda_key] = lambda_array

    def print_time_spectra_filename(self, time_spectra_filename):
        """display the folder and filename in the corresponding widgets"""
        time_spectra_filename = PurePath(time_spectra_filename)
        base_time_spectra = str(time_spectra_filename.name)
        folder_name = str(time_spectra_filename.parent)
        self.list_ui[self.data_type]["time_spectra"]["filename"].setText(base_time_spectra)
        self.list_ui[self.data_type]["time_spectra"]["folder"].setText(folder_name)
        self.parent.data_metadata[self.data_type]["time_spectra"]["folder"] = folder_name

    def retrieve_files(self, data_type="sample"):
        """
        type = ['sample', 'ob', 'normalized', 'time_spectra']
        """
        self.data_type = data_type

        mydialog = FileDialog()
        mydialog.setDirectory(self.parent.default_path[data_type])
        mydialog.exec_()

        selected_files = mydialog.filesSelected()

        if selected_files:
            if len(selected_files) == 1:
                if os.path.isdir(selected_files[0]):
                    self.load_directory(selected_files[0])
                else:
                    self.load_files(selected_files[0])
            else:
                self.load_files(selected_files)

            if (data_type == "sample") or (data_type == "normalized"):
                self.retrieve_time_spectra()
                self.load_time_spectra()

        else:
            self.user_canceled = True

        # calculate mean data array for normalization tab
        if data_type == "sample":
            _data = self.parent.data_metadata["sample"]["data"]
            normalization_mean_data = np.mean(_data, axis=0)
            self.parent.data_metadata["normalization"]["data"] = normalization_mean_data

    def retrieve_time_spectra(self):
        folder = self.parent.default_path[self.data_type]
        time_spectra_name_format = "*_Spectra.txt"
        [file_name, _] = QFileDialog.getOpenFileName(
            caption="Select the Time Spectra File",
            directory=folder,
            filter="Txt ({});;All (*.*)".format(time_spectra_name_format),
        )

        if file_name:
            folder_name = str(FileHandler.get_parent_path(file_name))
            base_file_name = str(FileHandler.get_base_filename(file_name))
            self.parent.time_spectra_folder = str(FileHandler.get_parent_folder(file_name))

            self.list_ui[self.data_type]["time_spectra"]["filename"].setText(base_file_name)
            self.list_ui[self.data_type]["time_spectra"]["folder"].setText(folder_name)
            self.parent.data_metadata[self.data_type]["time_spectra"]["folder"] = folder_name
            self.parent.data_metadata[self.data_type]["time_spectra"]["filename"] = file_name

            self.load_time_spectra(time_spectra_file=file_name)
            return True

        return False

    def load_directory(self, folder):
        list_files = glob.glob(folder + "/*.*")
        if len(list_files) == 0:
            raise TypeError
        image_type = self.get_image_type(list_files)
        o_load_image = LoadFiles(parent=self.parent, image_ext=image_type, folder=folder)
        self.populate_list_widget(o_load_image)
        self.parent.data_files[self.data_type] = o_load_image.list_of_files
        self.parent.data_metadata[self.data_type]["folder"] = o_load_image.folder
        # self.parent.sample_folder = os.path.dirname(o_load_image.folder)
        self.parent.sample_folder = o_load_image.folder
        self.parent.data_metadata[self.data_type]["data"] = o_load_image.image_array

    def populate_list_widget(self, o_loader):
        list_of_files = o_loader.list_of_files

        _list_ui = self.list_ui[self.data_type]["list"]
        _list_ui.clear()
        for _row, _file in enumerate(list_of_files):
            _item = QListWidgetItem(_file)
            _list_ui.insertItem(_row, _item)

        _folder = o_loader.folder
        self.folder = _folder
        self.parent.default_path[self.data_type] = _folder
        self.list_ui[self.data_type]["folder"].setText(os.path.basename(os.path.abspath(_folder)))

    @staticmethod
    def get_image_type(list_of_files):
        image_type = FileHandler.get_file_extension(list_of_files[0])
        return image_type


class FileDialog(QFileDialog):
    selected_files: list = []

    def __init__(self, *args):
        QFileDialog.__init__(self, *args)
        self.setOption(self.DontUseNativeDialog, False)
        self.setFileMode(self.ExistingFiles)

    def openClicked(self):
        indexes = self.tree.selectionModel().selectedIndexes()
        files = []
        for i in indexes:
            if i.column() == 0:
                files.append(os.path.join(str(self.directory().absolutePath()), str(i.data())))
        self.selected_files = files
        self.close()

    def filesSelected(self):
        return self.selected_files



#!/usr/bin/env python
"""
Export the strain mapping table to an ASCII or JSON file.
"""

import os

from loguru import logger
from qtpy.QtWidgets import QFileDialog

from ibeatles import DataType, FileType
from ibeatles.fitting.kropff.get import Get as KropffGet
from ibeatles.step6 import ParametersToDisplay
from ibeatles.utilities.export import format_kropff_dict, format_kropff_table
from ibeatles.utilities.file_handler import (
    FileHandler,
    create_full_export_file_name,
)


class Export:
    def __init__(self, parent=None, grand_parent=None):
        self.parent = parent
        self.grand_parent = grand_parent
        self.working_dir = os.path.dirname(
            os.path.abspath((self.grand_parent.data_metadata[DataType.normalized]["folder"]))
        )

    @staticmethod
    def _make_image_base_name(normalized_folder, ext="tiff", parameters=ParametersToDisplay.d):
        base_file_name = os.path.basename(normalized_folder) + "_" + parameters + f".{ext}"
        return base_file_name

    def ascii(self):
        self.export_table(file_type=FileType.ascii)

    def json(self):
        self.export_table(file_type=FileType.json)

    def export_table(self, file_type: FileType = FileType.ascii):
        output_folder = str(
            QFileDialog.getExistingDirectory(
                self.grand_parent,
                f"Select where to export the table as an {file_type} file",
                self.working_dir,
            )
        )

        if output_folder:
            self.export_with_specified_file_type(file_type=file_type, output_folder=output_folder)

    def export_with_specified_file_type(self, file_type: FileType = FileType.ascii, output_folder: str = None):
        # create output file  name
        output_folder = os.path.abspath(output_folder)
        # output_file_name = os.path.join(output_folder, "strain_mapping_table.txt")
        output_file_name = create_full_export_file_name(os.path.join(output_folder, "strain_mapping_table"), file_type)

        kropff_table_dictionary = self.grand_parent.kropff_table_dictionary

        o_get = KropffGet(parent=self.parent, grand_parent=self.grand_parent)
        strain_mapping_dict = o_get.strain_mapping_dictionary()

        if file_type == FileType.ascii:
            formatted_table = format_kropff_table(
                table=kropff_table_dictionary,
                d_dict=self.parent.d_dict,
                strain_dict=strain_mapping_dict,
            )
            FileHandler.make_ascii_file(data=formatted_table, output_file_name=output_file_name)
        else:
            formatted_dict = format_kropff_dict(
                table=kropff_table_dictionary,
                d_dict=self.parent.d_dict,
                strain_dict=strain_mapping_dict,
                json_friendly=True,
            )
            FileHandler.make_json_file(data_dict=formatted_dict, output_file_name=output_file_name)

        logger.info(f"Exported {file_type} strain mapping table: {output_file_name}")



import matplotlib
import matplotlib.pyplot as plt

matplotlib.use("Qt5Agg")


from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
from matplotlib.figure import Figure


class MplCanvas(FigureCanvasQTAgg):
    def __init__(self, parent=None, width=5, height=4, dpi=100):
        fig = Figure(figsize=(width, height), dpi=dpi)
        self.axes = fig.add_subplot(111)
        self.fig = fig
        super(MplCanvas, self).__init__(fig)


class MplCanvasColorbar(FigureCanvasQTAgg):
    def __init__(self, parent=None, width=5, height=4, dpi=100):
        # fig = Figure(figsize=(width, height), dpi=dpi)
        self.fig, self.axes = plt.subplots()

        # self.axes = fig.add_subplot(111)
        # self.fig = fig
        super(MplCanvasColorbar, self).__init__(self.fig)

        # self.fig = Figure(figsize=(width, height), dpi=dpi)
        #
        # self.axes = plt.subplot2grid((3, 3), (0, 0), colspan=2)
        # self.cax = plt.subplot2grid((3, 3), (0, 1))
        #
        # super(MplCanvasColorbar, self).__init__(self.fig)



#!/usr/bin/env python3
"""Utility functions for fitting module."""

import logging
from typing import Callable, Tuple

import numpy as np


def remove_invalid_data_points(xdata: np.ndarray, ydata: np.ndarray) -> Tuple[np.ndarray, np.ndarray]:
    """
    Remove NaN and infinite values from input data arrays.

    Parameters:
    -----------
    xdata : np.ndarray
        The x-axis data.
    ydata : np.ndarray
        The y-axis data.

    Returns:
    --------
    Tuple[np.ndarray, np.ndarray]
        A tuple containing the cleaned x and y data arrays.

    Warns:
    ------
    UserWarning
        If any data points were removed due to being NaN or infinite.
    """
    valid_indices = np.isfinite(xdata) & np.isfinite(ydata)
    if np.sum(valid_indices) < len(xdata):
        removed_count = len(xdata) - np.sum(valid_indices)
        logging.warning(f"Removed {removed_count} corrupted data point(s) from the input.")
        xdata = xdata[valid_indices]
        ydata = ydata[valid_indices]

    return xdata, ydata


def generate_synthetic_transmission(
    model: Callable,
    wavelengths: np.ndarray,
    true_params: dict,
    noise_level: float = 0.01,
) -> Tuple[np.ndarray, np.ndarray]:
    """
    Generate synthetic transmission data based on a given model with added noise.

    Parameters:
    -----------
    model : Callable
        The model function to use for generating synthetic data.
    wavelengths : np.ndarray
        Array of wavelength values.
    true_params : dict
        Dictionary of true parameter values for the model.
    noise_level : float, optional
        Standard deviation of Gaussian noise to add (default is 0.01).

    Returns:
    --------
    Tuple[np.ndarray, np.ndarray]
        Wavelengths and corresponding noisy transmission values
    """
    ideal_transmission = model(wavelengths, **true_params)
    # NOTE: add Gaussian noise (mu=0, sigma=noise_level) to the ideal transmission
    noise = np.random.normal(0, noise_level, len(wavelengths))
    noisy_transmission = ideal_transmission + noise
    return wavelengths, noisy_transmission



#!/usr/bin/env python
"""
Value Table Handler
"""

import numpy as np
from qtpy import QtGui
from qtpy.QtWidgets import QMenu

from ibeatles.fitting.march_dollase.advanced_selection_launcher import (
    AdvancedSelectionLauncher,
)
from ibeatles.fitting.march_dollase.export_fitting_handler import (
    ExportFittingHandler,
)

from ..table_dictionary.export import Export
from ..table_dictionary.table_dictionary_handler import TableDictionaryHandler
from .filling_table_handler import FillingTableHandler
from .set_fitting_variables_launcher import SetFittingVariablesLauncher


class ValueTableHandler(object):
    __advanced_selection = None
    __set_variables = None
    __select_all = None
    __unselect_all = None
    __reset = None
    __fixed = None
    __unfixed = None
    __export = None
    __import = None

    def __init__(self, grand_parent=None, parent=None):
        self.grand_parent = grand_parent
        self.parent = parent

    def right_click(self, position):
        menu = QMenu(self.grand_parent)

        if (len(self.grand_parent.fitting_ui.data) == 0) or (self.grand_parent.binning_line_view["pos"] is None):
            status = False
        else:
            status = True

        self.__select_all = menu.addAction("Select All")
        self.__select_all.setEnabled(status)
        self.__unselect_all = menu.addAction("Unselect All")
        self.__unselect_all.setEnabled(status)
        menu.addSeparator()

        self.__advanced_selection = menu.addAction("Selection/Lock Tool ...")
        self.__advanced_selection.setEnabled(status)

        self.__set_variables = menu.addAction("Variables Advanced Tool ...")
        self.__set_variables.setEnabled(status)

        # menu.addSeparator()
        #
        # self.__fixed = menu.addAction("Fixed Variables Selected")
        # self.__fixed.setEnabled(status)
        #
        # self.__unfixed = menu.addAction("Unfixed Variables Selected")
        # self.__unfixed.setEnabled(status)

        menu.addSeparator()

        self.__export = menu.addAction("Export Table ...")
        self.__export.setEnabled(True)

        menu.addSeparator()

        self.__export_fitting = menu.addAction("Export Fitting ...")
        self.__export_fitting.setEnabled(False)

        # _reset = menu.addAction("Full Reset")
        # _reset.setEnabled(status)
        # _reset.setEnabled(False) #remove once implemented

        action = menu.exec_(QtGui.QCursor.pos())
        if action is None:
            return

        funct = self.select_action(action)
        if funct is not None:
            funct()

    def select_action(self, action):
        """function to replace the complext if/elif/else statement
        of right_click()."""
        return {
            self.__select_all: self.select_all,
            self.__unselect_all: self.unselect_all,
            self.__advanced_selection: self.advanced_selection,
            self.__set_variables: self.set_variables,
            self.__reset: self.reset,
            self.__unfixed: self.unfixed_variables,
            self.__export: self.export_table,
            # self.__import: self.import_table,
            self.__export_fitting: self.export_fitting,
        }.get(action, lambda: None)()

    def export_fitting(self):
        o_export_fitting = ExportFittingHandler(grand_parent=self.grand_parent)
        o_export_fitting.run()

    def export_table(self):
        o_export = Export(parent=self.parent, grand_parent=self.grand_parent)
        o_export.run()

    def changed_fixed_variables_status(self, status=True):
        selection = self.parent.ui.value_table.selectedRanges()
        nbr_row = self.parent.ui.value_table.rowCount()

        o_fill_table = FillingTableHandler(grand_parent=self.grand_parent)
        row_to_show_status = o_fill_table.get_row_to_show_state()

        table_dictionary = self.grand_parent.march_table_dictionary

        column_variable_match = {
            5: "d_spacing",
            7: "sigma",
            9: "alpha",
            11: "a1",
            13: "a2",
            15: "a5",
            17: "a7",
        }

        for _select in selection:
            left_column = _select.leftColumn()
            right_column = _select.rightColumn()
            col_range = np.arange(left_column, right_column + 1)
            for _col in col_range:
                if _col < 5:
                    continue

                # only work with even col number
                if (_col % 2) == 0:
                    continue

                name_variable = column_variable_match.get(_col)

                for _index in np.arange(nbr_row):
                    _entry = table_dictionary[str(_index)]
                    if _entry["lock"]:
                        continue

                    if row_to_show_status == "all":
                        _entry[name_variable]["fixed"] = status
                        table_dictionary[str(_index)] = _entry

                    elif row_to_show_status == "active":
                        if _entry["active"]:
                            _entry[name_variable]["fixed"] = status
                            table_dictionary[str(_index)] = _entry

            self.grand_parent.march_table_dictionary = table_dictionary

            o_fitting = FillingTableHandler(grand_parent=self.grand_parent)
            o_fitting.fill_table()

    def fixed_variables(self):
        self.changed_fixed_variables_status(status=True)

    def unfixed_variables(self):
        self.changed_fixed_variables_status(status=False)

    def select_all(self):
        o_table = TableDictionaryHandler(grand_parent=self.grand_parent, parent=self.parent)
        o_table.select_full_table()

    def unselect_all(self):
        o_table = TableDictionaryHandler(grand_parent=self.grand_parent, parent=self.parent)
        o_table.unselect_full_table()

    def advanced_selection(self):
        AdvancedSelectionLauncher(grand_parent=self.grand_parent, parent=self.parent)

    def set_variables(self):
        SetFittingVariablesLauncher(grand_parent=self.grand_parent, parent=self.parent)

    def reset(self):
        print("reset")



#!/usr/bin/env python
"""
TOF combine export launcher
"""

import warnings

from qtpy.QtWidgets import QDialog

from ibeatles import DataType, load_ui
from ibeatles.tools.tof_combine.utilities.get import Get
from ibeatles.utilities.status_message_config import (
    StatusMessageStatus,
    show_status_message,
)

warnings.filterwarnings("ignore")


class TofCombineExportLauncher(QDialog):
    def __init__(self, parent=None, grand_parent=None):
        self.parent = parent
        self.grand_parent = grand_parent
        QDialog.__init__(self, parent=parent)
        self.ui = load_ui("ui_tof_combine_export.ui", baseinstance=self)

    def ok_clicked(self):
        o_get = Get(parent=self)
        data_type_selected = o_get.combine_export_mode()
        self.close()
        output_folder = self.parent.combine_run(data_type_selected=data_type_selected)
        if output_folder:
            self.parent.reload_run_in_main_ui(data_type_selected=data_type_selected, output_folder=output_folder)
            self.parent.close()

            message = f"TOF combined exported to {output_folder}"
            if not (data_type_selected == DataType.none):
                message += f" and loaded back in {data_type_selected}"
            message += "!"
            status = StatusMessageStatus.ready

        else:
            message = "User cancel export process!"
            status = StatusMessageStatus.warning

        show_status_message(parent=self.grand_parent, message=message, status=status, duration_s=10)



from qtpy import QtGui

LOCK_ROW_BACKGROUND = QtGui.QColor(75, 150, 150)
UNLOCK_ROW_BACKGROUND = QtGui.QColor(255, 255, 255)
REJECTED_ROW_BACKGROUND = QtGui.QColor(255, 255, 150)

ERROR_TOLERANCE = 100


class KropffThresholdFinder:
    sliding_average = "sliding_average"
    error_function = "error_function"
    change_point = "change_point"


class FittingKropffBraggPeakColumns:
    l_hkl_value = 2
    tau_value = 3
    sigma_value = 4
    l_hkl_error = 5
    tau_error = 6
    sigma_error = 7


class FittingKropffHighLambdaColumns:
    a0 = 2
    b0 = 3


class FittingKropffLowLambdaColumns:
    ahkl = 2
    bhkl = 3


class FittingRegions:
    high_lambda = "high_lambda"
    low_lambda = "low_lambda"
    bragg_peak = "bragg_peak"


class BraggPeakInitParameters:
    fix_flag = "fix flag"
    fix_value = "fix value"
    range_from = "range from"
    range_to = "range to"
    range_step = "range step"

    from_lambda = "from lambda"
    to_lambda = "to lambda"
    hkl_selected = "hkl selected"
    lambda_0 = "lambda 0"
    element = "element"


class SessionSubKeys:
    table_dictionary = "table dictionary"
    high_tof = "high_tof"
    low_tof = "low_tof"
    bragg_peak = "bragg_peak"

    fitted = "fitted"

    a0 = "a0"
    b0 = "b0"
    graph = "graph"
    ahkl = "ahkl"
    bhkl = "bhkl"
    lambda_hkl = "lambda_hkl"
    tau = "tau"
    sigma = "sigma"

    table_selection = "table selection"
    lock = "lock"
    automatic_bragg_peak_threshold_finder = "automatic bragg peak threshold finder"
    kropff_bragg_peak_good_fit_conditions = "kropff bragg peak good fit conditions"
    l_hkl = "l_hkl"
    l_hkl_error = "l_hkl_error"
    state = "state"
    value = "value"
    t_error = "t_error"
    sigma_error = "sigma_error"
    kropff_lambda_settings = "kropff lambda settings"
    fix = "fix"
    range = "range"
    bragg_peak_row_rejections_conditions = "bragg peak row conditions"
    less_than = "less_than"
    more_than = "more_than"
    automatic_fitting_threshold_width = "automatic fitting threshold width"
    automatic_bragg_peak_threshold_algorithm = "automatic bragg peak threshold algorithm"


class RightClickTableMenu:
    replace_values = "replace values by surrounding median values"
    display_fitting_parameters = "display the fitting parameters"
